@@ -10,6 +10,7 @@ use std::{borrow::Cow, io::Write};
 
 use anyhow::{anyhow, Result};
 use crossbeam_channel::{bounded, Receiver, RecvTimeoutError, Sender};
+use etcetera::BaseStrategy;
 use ignore::overrides::OverrideBuilder;
 use ignore::{self, WalkBuilder};
 use regex::bytes::Regex;
@@ -34,7 +35,10 @@ enum ReceiverMode {
 }
 
 /// The Worker threads can result in a valid entry having PathBuf or an error.
+#[allow(clippy::large_enum_variant)]
 pub enum WorkerResult {
+    // Errors should be rare, so it's probably better to allow large_enum_variant than
+    // to box the Entry variant
     Entry(DirEntry),
     Error(ignore::Error),
 }
@@ -44,12 +48,12 @@ pub const MAX_BUFFER_LENGTH: usize = 1000;
 /// Default duration until output buffering switches to streaming.
 pub const DEFAULT_MAX_BUFFER_TIME: Duration = Duration::from_millis(100);
 
-/// Recursively scan the given search path for files / pathnames matching the pattern.
+/// Recursively scan the given search path for files / pathnames matching the patterns.
 ///
 /// If the `--exec` argument was supplied, this will create a thread pool for executing
 /// jobs in parallel from a given command line and the discovered paths. Otherwise, each
 /// path will simply be written to standard output.
-pub fn scan(paths: &[PathBuf], pattern: Arc<Regex>, config: Arc<Config>) -> Result<ExitCode> {
+pub fn scan(paths: &[PathBuf], patterns: Arc<Vec<Regex>>, config: Arc<Config>) -> Result<ExitCode> {
     let first_path = &paths[0];
 
     // Channel capacity was chosen empircally to perform similarly to an unbounded channel
@@ -74,6 +78,7 @@ pub fn scan(paths: &[PathBuf], pattern: Arc<Regex>, config: Arc<Config>) -> Resu
         .git_ignore(config.read_vcsignore)
         .git_global(config.read_vcsignore)
         .git_exclude(config.read_vcsignore)
+        .require_git(config.require_git_to_read_vcsignore)
         .overrides(overrides)
         .follow_links(config.follow_links)
         // No need to check for supported platforms, option is unavailable on unsupported ones
@@ -85,26 +90,17 @@ pub fn scan(paths: &[PathBuf], pattern: Arc<Regex>, config: Arc<Config>) -> Resu
     }
 
     if config.read_global_ignore {
-        #[cfg(target_os = "macos")]
-        let config_dir_op = std::env::var_os("XDG_CONFIG_HOME")
-            .map(PathBuf::from)
-            .filter(|p| p.is_absolute())
-            .or_else(|| dirs_next::home_dir().map(|d| d.join(".config")));
-
-        #[cfg(not(target_os = "macos"))]
-        let config_dir_op = dirs_next::config_dir();
-
-        if let Some(global_ignore_file) = config_dir_op
-            .map(|p| p.join("fd").join("ignore"))
-            .filter(|p| p.is_file())
-        {
-            let result = walker.add_ignore(global_ignore_file);
-            match result {
-                Some(ignore::Error::Partial(_)) => (),
-                Some(err) => {
-                    print_error(format!("Malformed pattern in global ignore file. {}.", err));
+        if let Ok(basedirs) = etcetera::choose_base_strategy() {
+            let global_ignore_file = basedirs.config_dir().join("fd").join("ignore");
+            if global_ignore_file.is_file() {
+                let result = walker.add_ignore(global_ignore_file);
+                match result {
+                    Some(ignore::Error::Partial(_)) => (),
+                    Some(err) => {
+                        print_error(format!("Malformed pattern in global ignore file. {}.", err));
+                    }
+                    None => (),
                 }
-                None => (),
             }
         }
     }
@@ -150,7 +146,7 @@ pub fn scan(paths: &[PathBuf], pattern: Arc<Regex>, config: Arc<Config>) -> Resu
     let receiver_thread = spawn_receiver(&config, &quit_flag, &interrupt_flag, rx);
 
     // Spawn the sender threads.
-    spawn_senders(&config, &quit_flag, pattern, parallel_walker, tx);
+    spawn_senders(&config, &quit_flag, patterns, parallel_walker, tx);
 
     // Wait for the receiver thread to print out all results.
     let exit_code = receiver_thread.join().unwrap();
@@ -343,28 +339,23 @@ fn spawn_receiver(
             if cmd.in_batch_mode() {
                 exec::batch(rx, cmd, &config)
             } else {
-                let out_perm = Arc::new(Mutex::new(()));
+                let out_perm = Mutex::new(());
 
-                // Each spawned job will store it's thread handle in here.
-                let mut handles = Vec::with_capacity(threads);
-                for _ in 0..threads {
-                    let config = Arc::clone(&config);
-                    let rx = rx.clone();
-                    let cmd = Arc::clone(cmd);
-                    let out_perm = Arc::clone(&out_perm);
+                thread::scope(|scope| {
+                    // Each spawned job will store it's thread handle in here.
+                    let mut handles = Vec::with_capacity(threads);
+                    for _ in 0..threads {
+                        let rx = rx.clone();
 
-                    // Spawn a job thread that will listen for and execute inputs.
-                    let handle = thread::spawn(move || exec::job(rx, cmd, out_perm, &config));
+                        // Spawn a job thread that will listen for and execute inputs.
+                        let handle = scope.spawn(|| exec::job(rx, cmd, &out_perm, &config));
 
-                    // Push the handle of the spawned thread into the vector for later joining.
-                    handles.push(handle);
-                }
-
-                let exit_codes = handles
-                    .into_iter()
-                    .map(|handle| handle.join().unwrap())
-                    .collect::<Vec<_>>();
-                merge_exitcodes(exit_codes)
+                        // Push the handle of the spawned thread into the vector for later joining.
+                        handles.push(handle);
+                    }
+                    let exit_codes = handles.into_iter().map(|handle| handle.join().unwrap());
+                    merge_exitcodes(exit_codes)
+                })
             }
         } else {
             let stdout = io::stdout();
@@ -380,13 +371,13 @@ fn spawn_receiver(
 fn spawn_senders(
     config: &Arc<Config>,
     quit_flag: &Arc<AtomicBool>,
-    pattern: Arc<Regex>,
+    patterns: Arc<Vec<Regex>>,
     parallel_walker: ignore::WalkParallel,
     tx: Sender<WorkerResult>,
 ) {
     parallel_walker.run(|| {
         let config = Arc::clone(config);
-        let pattern = Arc::clone(&pattern);
+        let patterns = Arc::clone(&patterns);
         let tx_thread = tx.clone();
         let quit_flag = Arc::clone(quit_flag);
 
@@ -456,7 +447,10 @@ fn spawn_senders(
                 }
             };
 
-            if !pattern.is_match(&filesystem::osstr_to_bytes(search_str.as_ref())) {
+            if !patterns
+                .iter()
+                .all(|pat| pat.is_match(&filesystem::osstr_to_bytes(search_str.as_ref())))
+            {
                 return ignore::WalkState::Continue;
             }
 
